package org.jlibsedml;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.jdom.Namespace;
import org.jlibsedml.execution.IModelResolver;
import org.jlibsedml.execution.ModelResolver;
/**
 * The top level object in a SED-ML document.<p/>
 * To create a SedML object, create a SED-ML document first.<p/>
 * 
 * <p/>This class is basically a container element for the 5 different parts of the SED-ML - the model, simulation,
 * task description, data generator, and output.</p>
 * 
 * Elements can be added in two ways:<ul>
 * <li> Either pass in a previously created list, e.g.,
 *     <pre>
 *          List<Simulation> simulations = createListOfSimulationsSomewhereElse();
 *          sedml.setSimulations(simulations);
 *     </pre>
 *     or 
 *  <li> Add simulations one at a time:
 *   <pre>
 *          Simulation sim = createASimulation();
 *          sedml.addSimulation(sim);
 *    </pre>
 *  </ul>
 *  
 *  Elements can be added to a list that has previously been set, however setting in a list
 *   will overwrite any collections generated from either of the two approaches above. </p>
 *   
 *   All getListOfXXX() methods will return read-only lists. Attempting to modify these lists will generate runtime
 *   exceptions. To manipulate the lists, then, use the add/remove/set methods.<p/>
 *   
 *  <p> Elements can be searched for by the getXXXWithId() methods. E.g., </p>
 *  <pre>
 *       Model model = sedml.getModelWithId("myModel");
 *       if (model != null){
 *         // do something.
 *       }
 *  </pre>
 *  <p> It should be noted that all collections held in this object, and elsewhere in SEDML, are mutable
 *   from outside the object. So, if you pass in a collection of Simulations into this class, then
 *    modify the collection outside this object, then this collection  will be modified.</p>
 * @author anu/radams
 *
 */
public final class SedML extends SEDBase {

	@Override
	public String toString() {
		return "SedML [level=" + level + "]";
	}

	private int level = 1;
	private int version = 1;
	private Namespace xmlns = null;

	private List additionalNamespaces = new ArrayList();

	private List<Model> models = new ArrayList<Model>();
	private List<Simulation> simulations = new ArrayList<Simulation>();
	private List<Task> tasks = new ArrayList<Task>();
	private List<DataGenerator> dataGenerators = new ArrayList<DataGenerator>();
	private List<Output> outputs = new ArrayList<Output>();
	
	/**
     * Sorts a list of Outputs into the correct order specified in the schema. 
     * @author radams
     *
     */
    static class OutputComparator implements Comparator<Output>{
        static Map<String, Integer> changeKindOrder;
        static {
       changeKindOrder = new HashMap<String, Integer>();
       changeKindOrder.put(SEDMLTags.PLOT2D_KIND, 1);
       changeKindOrder.put(SEDMLTags.PLOT3D_KIND, 2);
       changeKindOrder.put(SEDMLTags.REPORT_KIND, 3);
  
        }
     public int compare(Output o1, Output o2) {
         return changeKindOrder.get(o1.getKind()).compareTo(
                 changeKindOrder.get  (o2.getKind()));
     }
        
    }

	 SedML(int aLevel, int aVersion, Namespace aNameSpace) {
		this.level = aLevel;
		this.version = aVersion;
		this.xmlns = aNameSpace;
	}

	  SedML(Namespace aNamespace) {
		this.xmlns = aNamespace;
		this.level = 1;
		this.version = 1;
	}

	/**
	 * Returns a read-only list of models in SedDocument
	 * @return
	 */
	public List<Model> getModels() {
		return Collections.unmodifiableList(models);
	}

	/**
	 * Returns  a read-only list of simulations in SedML
	 * @return
	 */
	public List<Simulation> getSimulations() {
		return Collections.unmodifiableList(simulations);
	}

	/**
	 * Returns a read-only  list of tasks in SedMl
	 * @return
	 */
	public List<Task> getTasks() {
		return Collections.unmodifiableList(tasks);
	}
	
	@Override
	public String getElementName() {
		return SEDMLTags.ROOT_NODE_TAG;
	}

	/**
	 * Returns a read-only list of data generators in SedML
	 * @return
	 */
	public List<DataGenerator> getDataGenerators() {
		return Collections.unmodifiableList(dataGenerators);
	}

	/**
	 * Returns a read-only list of outputs in SedDocument. This method  does not return the
	 *  list in the order by whic Outputs were added. Instead, it orders the outputs by types
	 *  to agree with the schema. I.e., Plot2D ,Plot3d, Reports. <br/
	 * @return A possibly empty but non-null <code>List</code> of Outputs.
	 */
	public List<Output> getOutputs() {
	    Collections.sort(outputs, new OutputComparator());
		return Collections.unmodifiableList(outputs);
	}

	/**
	 * Sets additional namespaces on SedDocument
	 */
	public void setAdditionalNamespaces(List additionalNamespaces) {
		this.additionalNamespaces = additionalNamespaces;
	}

	/**
	 * Sets list of models on SedDocument
	 */
	public void setModels(List<Model> models) {
		this.models = models;
	}
	
	/**
	 * Adds a {@link Model} to this object's list of Models, if not already present.
	 * @param model A non-null {@link Model} element
	 * @return <code>true</code> if model added, <code>false </code> otherwise.
	 */
	public boolean addModel(Model model){
		if(!models.contains(model))
			return models.add(model);
		return false;
	}
	
	/**
	 * Removes a {@link Model} from this object's list of Models.
	 * @param model A non-null {@link Model} element
	 * @return <code>true</code> if model removed, <code>false </code> otherwise.
	 */
	public boolean removeModel(Model model){
			return models.remove(model);

	}

	/**
	 * Sets list of simulations on SedDocument
	 * @return
	 */
	public void setSimulations(List<Simulation> simulations) {
		this.simulations = simulations;
	}
	
	/**
	 * Adds a {@link Simulation} to this object's list of Simulations, if not already present.
	 * @param simulation A non-null {@link Simulation} element
	 * @return <code>true</code> if simulation added, <code>false </code> otherwise.
	 */
	public boolean addSimulation(Simulation simulation){
		if(!simulations.contains(simulation))
			return simulations.add(simulation);
		return false;
	}
	
	/**
	 * Removes a {@link Simulation} from this object's list of Simulations.
	 * @param simulation A non-null {@link Simulation} element
	 * @return <code>true</code> if simulation removed, <code>false </code> otherwise.
	 */
	public boolean removeSimulation(Simulation simulation){
		return simulations.remove(simulation);
	}

	/**
	 * Sets list of tasks on SedDocument
	 * @return
	 */
	public void setTasks(List<Task> tasks) {
		this.tasks = tasks;
	}

	/** Adds a {@link Task} to this object's list of Tasks, if not already present.
	 * @param task A non-null {@link Task} element
	 * @return <code>true</code> if task added, <code>false </code> otherwise.
	 */
	public boolean addTask(Task task){
		if(!tasks.contains(task))
			return tasks.add(task);
		return false;
	}
	
	/**
	 * Removes a {@link Task} from this object's list of Tasks.
	 * @param task A non-null {@link Task} element
	 * @return <code>true</code> if task removed, <code>false </code> otherwise.
	 */
	public boolean removeTask(Task task){
		return tasks.remove(task);
	}
	
	/**
	 * Sets list of data generators on SedDocument
	 * @return
	 */
	public void setDataGenerators(List<DataGenerator> dataGenerators) {
		this.dataGenerators = dataGenerators;
	}
	
	/** Adds a {@link DataGenerator} to this object's list of DataGenerators, if not already present.
	 * @param dataGenerator A non-null {@link DataGenerator} element
	 * @return <code>true</code> if dataGenerator added, <code>false </code> otherwise.
	 */
	public boolean addDataGenerator(DataGenerator dataGenerator){
		if(!dataGenerators.contains(dataGenerator))
			return dataGenerators.add(dataGenerator);
		return false;
	}
	
	/**
	 * Removes a {@link DataGenerator} from this object's list of DataGenerators.
	 * @param dg A non-null {@link DataGenerator} element
	 * @return <code>true</code> if  removed, <code>false </code> otherwise.
	 */
	public boolean removeDataGenerator(DataGenerator dg){
		return dataGenerators.remove(dg);
	}

	/**
	 * Sets list of outputs on SedDocument
	 * @return
	 */
	public void setOutputs(List<Output> outputs) {
		this.outputs = outputs;
	}
	
	/** Adds a {@link Output} to this object's list of Outputs, if not already present.
	 * @param output A non-null {@link Output} element
	 * @return <code>true</code> if dataGenerator added, <code>false </code> otherwise.
	 */
	public boolean addOutput(Output output){
		if(!outputs.contains(output))
			return outputs.add(output);
		return false;
	}
	
	/**
	 * Removes a {@link Output} from this object's list of Outputs.
	 * @param output A non-null {@link Output} element
	 * @return <code>true</code> if  removed, <code>false </code> otherwise.
	 */
	public boolean removeOutput(Output output){
		return outputs.remove(output);
	}

	

	
	
	/**
	 * Returns the SED-ML {@link Model} with id 'modelRef', if present. <br/>
	 * This method does not  return the content of the model; use implementations of {@link IModelResolver}
	 *  together with  the {@link ModelResolver} class to get the model content.
	 * @param  modelRef A non-null model Identifier
	 * @return The model element with the specified ID, or <code>null</code> if not found.
	 */
	public Model getModelWithId(String modelRef) {
		for (int i = 0; i < models.size(); i++) {
			Model m = models.get(i);
			if (m.getId().equals(modelRef)) {
				return m;
			}
		}
		return null;
	}

	/**
	 * Boolean test for whether the <code> modelRef</code> argument corresponds to a model ID defined in the list of models.
	 * @param modelRef
	 * @return <code>true</code> if a model is defined with <code> modelRef</code> argument, <code>false</code> otherwise.
	 */
	public boolean isModel(String modelRef) {
		for (int i = 0; i < models.size(); i++) {
			Model m = models.get(i);
			if (m.getId().equals(modelRef)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Returns simulation with id 'simRef', if present.
	 * @param simRef
	 * @return A {@link Simulation} object, or <code>null</code> if not found.
	 */
	public Simulation getSimulation(String simRef) {
		for (int i = 0; i < simulations.size(); i++) {
			Simulation s = simulations.get(i);
			if (s.getId().equals(simRef)) {
				return s;
			}
		}
		return null;
	}

	/**
	 * Returns task with id 'taskRef', if present
	 * @param taskRef
	 * @return A {@link Task} object, or <code>null</code> if not found.
	 */
	public Task getTaskWithId(String taskRef) {
		for (int i = 0; i < tasks.size(); i++) {
			Task t = tasks.get(i);
			if (t.getId().equals(taskRef)) {
				return t;
			}
		}
		return null;
	}

	/**
	 * Returns a {@link DataGenerator} with id 'dataGenRef', if present.
	 * @param dataGenRef
	 * @return A {@link DataGenerator} object, or <code>null</code> if not found.
	 */
	public DataGenerator getDataGeneratorWithId(String dataGenRef) {
		for (int i = 0; i < dataGenerators.size(); i++) {
			DataGenerator d = dataGenerators.get(i);
			if (d.getId().equals(dataGenRef)) {
				return d;
			}
		}
		return null;
	}
	
	/**
	 * Returns the {@link Output} with id 'outputID', if present.
	 * @param outputID
	 * @return A {@link Output} object, or <code>null</code> if not found.
	 */
	public Output getOutputWithId(String outputID) {
		for (int i = 0; i < outputs.size(); i++) {
			Output d = outputs.get(i);
			if (d.getId().equals(outputID)) {
				return d;
			}
		}
		return null;
	}

	/**
	 * Returns list of additional namespaces in SedDocument
	 * @return
	 */
	public List getAdditionalNamespaces() {
		return additionalNamespaces;
	}

	/**
	 * Returns version of SedDocument
	 * @return
	 */
	public int getVersion() {
		return version;
	}

	/**
	 * Returns level of SedDocument
	 * @return
	 */
	public int getLevel() {
		return level;
	}

	/**
	 * Returns default namespace of SedDocument
	 * @return
	 */
	public Namespace getNamespace() {
		return xmlns;
	}
	
	/**
	 * This method returns a non-redundant <code>List</code> of models which refer to external sources
	 *  and do not refer to other models in this document.
	 * @return Returns a possibly empty but non-null read-only list of base models in this document.
	 */
	public List<Model> getBaseModels(){
		Set<Model> set = new HashSet<Model>();
		for (Model m: getModels()){
			if(getModelWithId(m.getSource())==null){
				set.add(m);
			}
		}
		List<Model> rc = new ArrayList<Model>();
		for (Model m: set){
			rc.add(m);
		}
		return Collections.unmodifiableList(rc);
		
	}

    @Override
    public boolean accept(SEDMLVisitor visitor) {
           if(!visitor.visit(this)){
               return false;
           }
           for (Simulation sim:getSimulations()){
               if(!sim.accept(visitor)){
                   return false;
               }
           }
           for (Task t:getTasks()){
               if(!t.accept(visitor)){
                   return false;
               }
           }
           for (Model m:getModels()){
               if(!m.accept(visitor)){
                   return false;
               }
           } 
           for (DataGenerator dg:getDataGenerators()){
               if(!dg.accept(visitor)){
                   return false;
               }
           }
           for (Output o:getOutputs()){
               if(!o.accept(visitor)){
                   return false;
               }
           }
           return true;
        }
    

	

}

